<?php
Namespace App\Model\Admin;

use Symfony\Component\Translation\TranslatorInterface;
use App\Utils\Utils;

/**
 * Provides admin functions for Homegear gateway management
 *
 */
class HomegearGateway
{
    public $hg;
    private $gatewayList;
    private $translator;
    private $utils;

    public function __construct(TranslatorInterface $translator, Utils $utils)
    {
        $this->hg = new \Homegear\Homegear;
        $this->translator = $translator;
        $this->utils = $utils;
    }

    public function list()
    {
        $interfaces = array();
        $families = $this->hg->listFamilies();
        foreach ($families as $family) {
            $info = $this->hg->getPairingInfo($family['ID']);
            if (array_key_exists('interfaces',$info)) {
                $gateways = $this->hg->listInterfaces($family['ID']);
                foreach ($gateways as $k => $v) {
                    $gateways[$k]['FAMILYNAME'] = $family['NAME'];
                }
                $interfaces[$family['ID']] = $gateways;
            }
        }
        return $interfaces;
    }

    public function listFamilies()
    {
        $types = array();
        $families = $this->hg->listFamilies();
        foreach ($families as $family) {
            $info = $this->hg->getPairingInfo($family['ID']);
            if (array_key_exists('interfaces',$info)) {
                if (count($info['interfaces']) == 1 && array_key_exists('eventserver', $info['interfaces'])) { continue; }
                $types[] = $family;
            }
        }
        return $types;
    }

    public function listTypes($familyId = null)
    {
        if ($familyId === null) { return; }
        $notUsed = ['cc1100', 'coc', 'cul', 'cunx', 'hm-mod-rpi-pcb', 'tcm310', 'serial'];
        $isOSS = (substr($_SERVER['REQUEST_URI'],0,6) !== '/admin' ? false : true );
        $settings = $this->hg->getPairingInfo(intval($familyId));
        if (!array_key_exists('interfaces',$settings)) {
            return;
        }
        foreach ($settings['interfaces'] as $key => $setting) {
            if (!$isOSS && in_array($key,$notUsed)) { continue; }
            $data[] = ['id' => $key, 'name' => $this->translator->trans($setting['name'],array(),'admin')];
        }
        return $data;
    }

    public function getSettingParameters($familyId = null, $typeId = null)
    {
        $data = array();
        if ($familyId === null) { return; }
        $settings = $this->hg->getPairingInfo(intval($familyId));
        if (array_key_exists('searchInterfaces',$settings) && $settings['searchInterfaces'] === true) {
            $data[0]['type'] = 'searchInterface';
        }
        if ($typeId == 'homegeargateway') {
            $data[0]['type'] = 'searchGateway';
        }
        if (array_key_exists('interfaces',$settings)) {
            foreach ($settings['interfaces'] as $mykey => $setting) {
                if ($mykey == $typeId) {
                    $data[1]['type'] = 'createInterface';
                    foreach ($setting as $key => $item) {
                        if (is_array($item) && array_key_exists('label',$item)) {
                            $item['key'] = $key;
                            $item['description'] = $typeId.'.'.$item['label'].'.description';
                            $data[1]['settings'][] = $item;
                        }
                    }
                }
            }
            usort($data[1]['settings'], function($a, $b) { return $a['pos'] <=> $b['pos']; });
        }
        return $data;
    }
    
    /**
     * Search for gateways of a family
     * 
     * @return none
     */
    public function search($familyId = null)
    {
        if ($familyId === null) { return; }
        $result = $this->hg->searchInterfaces(intval($familyId));
        if ($result > 0) {
            return ['result' => 'success', 'message' => $result];
        }
        return ['result' => 'error', 'message' => $this->translator->trans("ajax.exception.gateway.notfound",array(),'admin')];
    }

    /**
     * Search for homegear-gateways of a family
     * 
     * @return array()
     */
    public function ssdpSearch($familyId = null)
    {
        $result = array();
        if ($familyId === null) { return; }
        $gateways = $this->hg->ssdpSearch("urn:schemas-upnp-org:device:basic:1", 5000);
        foreach($gateways as $gateway) {
            if (!array_key_exists("additionalFields", $gateway) || !array_key_exists("hg-family-id", $gateway["additionalFields"]) || !array_key_exists("hg-gateway-configured", $gateway["additionalFields"])) { continue; }
            if ($gateway["additionalFields"]["hg-family-id"] != $familyId || $gateway["additionalFields"]["hg-gateway-configured"] != "0") { continue; }
            $result[] = ['ip' => $gateway['ip'], 'id' => explode(':',$gateway['additionalFields']['usn'])[1]];
        }
        if (count($result) > 0) {
            return ['result' => 'success', 'data' => $result];
        }
        return ['result' => 'error', 'message' => $this->translator->trans("ajax.exception.gateway.notfound",array(),'admin')];
    }

    /**
     * Create a gateway for a family
     * 
     * @return none
     */
    public function create($familyId = null, $params = array())
    {
        if ($familyId === null || $params === null || !is_array($params)) { return; }
        // get the settings for the family id
        $settings = $this->hg->getPairingInfo(intval($familyId));
        if (array_key_exists('interfaces',$settings) && array_key_exists($params['gatewayType'],$settings['interfaces'])) {
            $data['devicetype'] = $params['gatewayType'];
            foreach ($settings['interfaces'][$params['gatewayType']] as $key => $item) {
                if (is_array($item)) {
                    // check if field is not requered and not set
                    if (array_key_exists('required',$item) && $item['required'] == false && $params[$key] == '') {
                        continue;
                    }
                    if (array_key_exists('label',$item) && array_key_exists($key,$params) && $params[$key] != '') {
                        if ($item['type'] == "boolean") {
                            if ($params[$key] == "true" || $params[$key] == 1) {
                                $data[$key] = intval(1);
                            } else {
                                $data[$key] = intval(0);
                            }
                        } elseif ($item['type'] == "integer") {
                            $data[$key] = intval($params[$key]);
                        } else {
                            $data[$key] = $params[$key];
                        }
                    } elseif (array_key_exists('const',$item)) {
                        if ($item['type'] == "boolean") {
                            if ($item['const'] == "true" || $item['const'] == 1) {
                                $data[$key] = intval(1);
                            } else {
                                $data[$key] = intval(0);
                            }
                        } elseif ($item['type'] == "integer") {
                            $data[$key] = intval($item['const']);
                        } else {
                            $data[$key] = $item['const'];
                        }
                    } else {
                        return ['result' => 'error', 'message' => $this->translator->trans("ajax.exception.gateway.$key.missing",array(),'admin')];
                    }
                }
            }
            // check if we want to add a homegear gateway
            if (array_key_exists('devicetype', $data) && $data['devicetype'] == 'homegeargateway') {
                $data['useIdForHostnameVerification'] = intval(1);
                $result = $this->_configureGateway($data);
                if (is_array($result) && array_key_exists('result',$result) && $result['result'] == 'error') {
                    if (array_key_exists('message',$result)) {
                        return $result;
                    } else {
                        return ['result' => 'error', 'message' => $this->translator->trans("ajax.exception.gateway.creation.failed",array(),'admin')];
                    }
                }
                // we don't need the password anymore
                unset($data['password']);
            }
            // everything is ok, so lets create the gateway interface
            // check if we have an id ...
            if (array_key_exists('id',$data)) {
                foreach ($data as $key => $value) {
                    // we don't need to set the id
                    if ($key == 'id') { continue; }
                    try {
                        $this->hg->setFamilySetting(intval($familyId),$data['id'].".$key", $value);
                    } catch(\Homegear\HomegearException $e) {
                        return ['error' => "Exception catched. Code: ".$e->getCode().". Message: ".$e->getMessage()];
                    }
                }
                // reload the desired module to activate the gateway
                $allModules = $this->hg->listModules();
                foreach ($allModules as $key => $module) {
                    if ($module['FAMILY_ID'] == $familyId) {
                        $this->hg->reloadModule($module['FILENAME']);
                    }
                }
            }
            return ['result' => 'success', 'message' => "1"];
        }
    }

    public function getSettings($familyId = null, $id = null)
    {
        $interfaces = $this->hg->listInterfaces(intval($familyId));
        foreach ($interfaces as $interface) {
            if ($interface['ID'] == $id) {
                $settings['interface'] = $interface;
            }
        }
        $data = $this->hg->getPairingInfo(intval($familyId));
        if (array_key_exists('interfaces',$data) && array_key_exists($settings['interface']['TYPE'], $data['interfaces'])) {
            foreach ($data['interfaces'][$settings['interface']['TYPE']] as $key => $setting) {
                if (is_array($setting)) {
                    foreach ($setting as $key => $value) {
                        if (is_array($value)) {
                            if ($key == 'id') { continue; }
                            try {
                                $settings['db'][] = $this->hg->getFamilySetting(intval($familyId),$id.".$key");
                            } catch(\Homegear\HomegearException $e) {
                                $settings['db'] = 'notFound';
                            }
                        }
                    }
                }
            }
        }
        if (array_key_exists('pairingMethods',$data) && array_key_exists('searchDevices',$data['pairingMethods'])) {
            $settings['searchDevices'] = true;
        }
        if (array_key_exists('deviceInfoFileUpload',$data)) {
            $settings['deviceInfoFileUpload'] = true;
        }
        if (array_key_exists('deviceInfoFileSuffix',$data)) {
            $settings['deviceInfoFileSuffix'] = $data['deviceInfoFileSuffix'];
        }
        return $settings;
    }

    public function delete($familyId = null, $id = null)
    {
        if ($familyId === null || $id === null) { return; }
        $interfaces = $this->hg->listInterfaces(intval($familyId));
        foreach ($interfaces as $interface) {
            if ($interface['ID'] == $id) {
                // automatically remove '-auto' from interface
                $settings['interface'] = str_replace('-auto','',$interface);
                // Backward compatibility: rename type, since ccu2 is now called ccu
                $settings['interface']['TYPE'] = str_replace('ccu2','ccu',$settings['interface']['TYPE']);
            }
        }
        $data = $this->hg->getPairingInfo(intval($familyId));
        if (array_key_exists('interfaces',$data) && array_key_exists($settings['interface']['TYPE'], $data['interfaces'])) {
            if ($settings['interface']['TYPE'] == 'homegeargateway') {
                $this->hg->managementDeleteCert($id);
            }
            $this->hg->deleteFamilySetting(intval($familyId),$id.".devicetype");
            foreach ($data['interfaces'][$settings['interface']['TYPE']] as $key => $setting) {
                if (is_array($setting)) {
                    if ($key == 'id') { continue; }
                    $this->hg->deleteFamilySetting(intval($familyId),$id.".$key");
                }
            }
            // reload the desired module to deactivate the gateway
            $allModules = $this->hg->listModules();
            foreach ($allModules as $key => $module) {
                if ($module['FAMILY_ID'] == $familyId) {
                    $this->hg->reloadModule($module['FILENAME']);
                }
            }
        }
    }

    public function searchDevices($id = null)
    {
        if ($id === null) { return; }
        try {
            $result['success'] = $this->hg->searchDevices(intval($id));
        } catch(\Homegear\HomegearException $e) {
            $result['exception'] = $e->getMessage();
        }
        return $result;
    }

    public function uploaddeviceInfoFile($familyId = null, $file = null)
    {
        if ($familyId === null || $file === null) { return; }
        $filename = $file->getClientOriginalName();
        $extension = pathinfo($filename, PATHINFO_EXTENSION);
        $data = $this->hg->getPairingInfo(intval($familyId));
        if (array_key_exists('deviceInfoFileSuffix',$data) && $data['deviceInfoFileSuffix'] == ".".$extension) {
            // file seems to be valid, so copy it!
            rename($file->getPathname(), "/tmp/$filename");
            $this->hg->managementCopyDeviceDescriptionFile("/tmp/$filename", intval($familyId));
            return 1;
        }
        return;
    }

    private function _configureGateway($data = null)
    {
        if ($data === null) { return ['result' => 'error']; }
        if (array_key_exists('id',$data) && array_key_exists('host',$data) && array_key_exists('password',$data)) {
            try {
                $id = $this->hg->managementCreateCert($data['id']);
            } catch(\Homegear\HomegearException $e) {
                return ['result' => 'error', 'message' => $this->translator->trans('ajax.exception.gateway.'.$e->getMessage(),array(),'admin')];
            }
            $caResult = $this->utils->getCommandStatus($id);
            if ($caResult['exitCode'] == 0) {
                try {
                    $this->hg->configureGateway($data['host'], 2018, file_get_contents('/etc/homegear/ca/cacert.pem'), file_get_contents('/etc/homegear/ca/certs/'.$data['id'].'.crt'), file_get_contents('/etc/homegear/ca/private/'.$data['id'].'.key'),$data['password']);
                } catch(\Homegear\HomegearException $e) {
                    $this->hg->managementDeleteCert($data['id']);
                    return ['result' => 'error', 'message' => $this->translator->trans('ajax.exception.gateway.connection',array(),'admin')];
                }
            } else {
                $result = $this->hg->managementDeleteCert($data['id']);
                return ['result' => 'error', 'message' => $this->translator->trans('ajax.exception.gateway.error.creating.certificate',array(),'admin')];        
            }
            return;
        }
        return ['result' => 'error'];
    }
}