<?php
Namespace App\Model\Admin;

use App\Model\HomegearHistory as BaseHistory;

/**
 * Provides admin functions for Homegear history management
 *
 */
class HomegearHistory extends BaseHistory
{
    private $measurements = null;
    private $defaultColor = "#1d71b8";

    public function __construct()
    {
        parent::__construct();
    }

    /**
     * Add a measurement for a device/channel
     * 
     * @return bool
     */
    public function addMeasurement($id = null, $channel = null, $variable = null)
    {
        // add a measurement for a specific item
        if ($id == 0 && $channel == -1) {
            // this is a system variable
            $init = $this->hg->getSystemVariable($variable);
        } else {
            // this is a device
            $init = $this->hg->getValue(intval($id), intval($channel), $variable);
        }
        try {
            $this->hg->influxdbSetLogging(intval($id),intval($channel),$variable,$init,true);
        } catch(\Homegear\HomegearException $e) {
            return $e;
        }
        return true;
    }

    /**
     * Get all measurements for a specific device
     * 
     * @return array()
     */
    public function getMeasurementsByDevice($id = null, $var = null) {
        // get all measurements for a specific item
        if ($id === null) { return; }
        // check if homegear-influxdb is available
        if (!$this->checkInfluxDb()) { return; }
        // re-format the variable if set
        if ($var !== null) { $var = preg_replace('/[^A-Za-z0-9 ]/', '', $var); }
        try {
            $measurements = $this->hg->influxdbGetLoggedVariables(intval($id));
        } catch(\Homegear\HomegearException $e) {
            return;
        }
        // we want a system variable
        if ($id == 0 && $var !== null) {
            foreach ($measurements as $key => $value) {
                if ($value['VARIABLE'] == $var) {
                    $measurements = $measurements[$key];
                } else {
                    unset($measurements[$key]);
                }
            }
        }
        return ((isset($measurements) && count($measurements) > 0) ? $measurements : null);
    }

    /**
     * delete all graphs and measurements for a specific device
     * 
     * @return none
     */
    public function deleteAllByItem($id = null, $var = null)
    {
        // get all measurements for item
        $all = $this->getMeasurementsByDevice($id, $var);
        if (is_array($all)) {
            foreach ($all as $single) {
                // delete all graph settings for this measurement
                $this->deleteGraph('history_'.$id.'_'.$single['channel'].'_'.$single['variable']);
                // delete measurement
                $this->deleteMeasurement($id,$single['channel'],$single['variable']);
            }
        }
    }

    /**
     * Create a graph for measurement
     * 
     * @return none
     */
    public function createGraph($id = null, $name = null, $datatype = null, $graphtype = null, $color = null, $sort = null)
    {
        if (($id === null) || ($name === null) || ($datatype === null) || ($graphtype === null)) { return; }
        if ($sort == null) { $sort = 0; }
        $data['name'] = $name;
        $data['type'] = $graphtype;
        $data['datatype'] = $datatype;
        $data['sort'] = $sort;
        $data['color'] = ($color === null ? $this->defaultColor : $color);
        list($data['color_r'],$data['color_g'],$data['color_b']) = $this->_hex2rgb($color);
        switch ($data['datatype']){
            case "Temperature":
                $data['labelY'] = "°C";
            break;
            case "Humidity":
            case "State (%)":
                $data['labelY'] = "%";
            break;
            default:
                $data['labelY'] = '';
            break;
        }
        $this->hg->setData(parent::componentGraphConfig,$id,json_encode($data));
    }

    /**
     * Update a graph
     * 
     * @return none
     */
    public function updateGraph($id = null, $name = null, $datatype = null, $graphtype = null, $color = null, $sort = null)
    {
        if (($id === null) || ($name === null) || ($datatype === null) || ($graphtype === null)) { return; }
        // delete first
        $this->hg->deleteData(parent::componentGraphConfig,$id);
        $this->createGraph($id, $name, $datatype, $graphtype, $color, $sort);
    }

    /**
     * Delete a graph
     * 
     * @return bool
     */
    public function deleteGraph($id = null) {
        if ($id === null) { return; }
        // delete current graph from all groups
        $allGroups = $this->loadGroups();
        if (count($allGroups) > 0) {
            foreach ($allGroups as $group) {
                $this->deleteGraphFromGroup($group['id'],$id);
            }
        }
        // delete graph
        try {
            $this->hg->deleteData(parent::componentGraphConfig,$id);
        } catch(\Homegear\HomegearException $e) {
            return $e;
        }
        return true;
    }

    /**
     * Load all graph groups
     * 
     * @return array()
     */
    public function loadGroups()
    {
        $groups = $this->hg->getData(parent::componentGraphGroups);
        if (is_array($groups) && (count($groups) > 0)) {
            foreach ($groups as $k => $v) {
                $current = json_decode($v,true);
                $data[] = ['id' => $k, 'name' => $current['name'], 'sort' => $current['sort']];
            }
            return $data;
        }
    }

    /**
     * Load a single graph group
     * 
     * @return array()
     */
    public function getGroupById($id = null)
    {
        if ($id === null) { return; }
        $data = json_decode($this->hg->getData(parent::componentGraphGroups,$id),true);
        $data['id'] = $id;
        // get graph-data
        if (is_array($data['graphs']) && (count($data['graphs']) > 0)) {
            foreach ($data['graphs'] as $k => $v) {
                // ok, something went totally wrong here
                if (is_array($v['id'])) { continue; }
                $sort = $data['graphs'][$k]['sort'];
                $data['graphs'][$k] = $this->getGraphConfig($v['id']);
                $data['graphs'][$k]['sort'] = $sort;
            }
        }
        return $data;
    }

    /**
     * Create a new graph group
     * 
     * @return string
     */
    public function createGroup($name = null)
    {
        if ($name === null) { return; }
        $id = md5($name);
        $data['name'] = $name;
        $data['sort'] = "0";
        $data['graphs'] = [];
        $this->hg->setData(parent::componentGraphGroups,$id,json_encode($data));
        return $id;
    }

    /**
     * Update a graph group
     * 
     * @return none
     */
    public function updateGroup($id = null, $name = null)
    {
        if ($id === null) { return; }
        $current = json_decode($this->hg->getData(parent::componentGraphGroups,$id),true);
        $current['name'] = $name;
        $this->hg->setData(parent::componentGraphGroups,$id,json_encode($current));
    }

    /**
     * Delete a graph group
     * 
     * @return none
     */
    public function deleteGroup($id = null)
    {
        if ($id === null) { return; }
        $this->hg->deleteData(parent::componentGraphGroups,$id);
    }

    /**
     * Update the group sorting
     * 
     * @return none
     */
    public function updateGroupSort($id = null, $position = null)
    {
        if (($id === null) || ($position === null)) { return; }
        $current = json_decode($this->hg->getData(parent::componentGraphGroups,$id),true);
        $current['sort'] = $position;
        $this->hg->setData(parent::componentGraphGroups,$id,json_encode($current));
    }

    /**
     * Add graphs to a group
     * 
     * @return none
     */
    public function addGraphsToGroup($id = null, $graphs = null)
    {
        if ($id === null) { return; }
        $current = $this->getGroupById($id);
        if (is_array($graphs) && (count($graphs) > 0)) {
            foreach ($graphs as $graph) {
                $current['graphs'][] = ['id' => $graph, 'sort' => 0];
            }
            $this->hg->setData(parent::componentGraphGroups,$id,json_encode($current));
        }
    }

    /**
     * Delete a graph from group
     * 
     * @return none
     */
    public function deleteGraphFromGroup($id = null, $graphId = null)
    {
        if ($id === null || $graphId === null) { return; }
        $current = json_decode($this->hg->getData(parent::componentGraphGroups,$id),true);
        // check if the current graph exists in group
        $key = array_search($graphId, array_column($current['graphs'], 'id'));
        if (isset($key)) {
            // graph exists!
            unset($current['graphs'][$key]);
            $current['graphs'] = array_values($current['graphs']);
            $this->hg->setData(parent::componentGraphGroups,$id,json_encode($current));
        }
    }

    public function updateGraphSort($id = null, $graphId = null, $position = null)
    {
        if (($id === null) || ($graphId === null) || ($position === null)) { return; }
        $current = json_decode($this->hg->getData(parent::componentGraphGroups,$id),true);
        // check if the current graph exists in group
        $key = array_search($graphId, array_column($current['graphs'], 'id'));
        if (isset($key)) {
            // graph exists!
            $current['graphs'][$key]['sort'] = $position;
            $this->hg->setData(parent::componentGraphGroups,$id,json_encode($current));
        }
    }

    /**
     * Delete a specific measurement
     * 
     * @return bool
     */
    public function deleteMeasurement($id = null, $channel = null, $variable = null) {
        // check if the current measurement has a configured graph
        $myVar = preg_replace('/_/','',$variable);
        $graphId = 'history_'.$id.'_'.$channel.'_'.$myVar;
        $config = $this->getGraphConfig($graphId);
        if ($config != '' && array_key_exists('id',$config)) {
            // found a configured graph, delete it (will automatically deleted from all groups, too!)
            $this->deleteGraph($graphId);
        }
        try {
            $this->hg->influxdbSetLogging(intval($id),intval($channel),$variable,'',false);
        } catch(\Homegear\HomegearException $e) {
            return $e;
        }
        return true;
    }

    /**
     * Convert hex to rgb
     * 
     * @return array()
     */
    private function _hex2rgb($hex)
    {
        $hex = str_replace("#", "", $hex);
        if(strlen($hex) == 3) {
           $r = hexdec(substr($hex,0,1).substr($hex,0,1));
           $g = hexdec(substr($hex,1,1).substr($hex,1,1));
           $b = hexdec(substr($hex,2,1).substr($hex,2,1));
        } else {
           $r = hexdec(substr($hex,0,2));
           $g = hexdec(substr($hex,2,2));
           $b = hexdec(substr($hex,4,2));
        }
        $rgb = array($r, $g, $b);
        //return implode(",", $rgb); // returns the rgb values separated by commas
        return $rgb; // returns an array with the rgb values
     }

}