<?php
Namespace App\Model\Admin;

Use App\Utils\Utils;
use Symfony\Component\Translation\TranslatorInterface;

/**
 * Provides admin functions for Homegear service management
 *
 */
class HomegearService
{
    public $hg;
    private $translator;
    private $utils;

    public function __construct(TranslatorInterface $translator, Utils $utils)
    {
        $this->hg = new \Homegear\Homegear;
        $this->utils = $utils;
        $this->translator = $translator;
    }

    /**
     * Check service list
     * 
     * @return array()
     */
    public function list($serviceList = null)
    {
        $data = [];
        if ($serviceList === null || !is_array($serviceList)) { return; }
        // check if there is a CloudMatic Account. If not, remove openvpn from array
        $cm = $this->hg->getData('CloudMatic');
        if (!is_array($cm) || !isset($cm['userid'])) {
            if (($key = array_search('openvpn@cloudmatic', $serviceList)) !== false) {
                unset($serviceList[$key]);
            }
        }
        $mgmt = $this->utils->checkManagementRunning(false);
        foreach ($serviceList as $service) {
            // check if service is up and running
            if ($mgmt) {
                $id = $this->hg->managementServiceCommand($service, 'status');
                $result = $this->utils->getCommandStatus($id);
                if (array_key_exists('output',$result) && $result['output'] != '') {
                    $data[] = ['name' => $service, 'installed' => 1, 'loaded' => (stristr($result['output'],'Loaded: loaded') ? 1 : 0), 'running' => (stristr($result['output'],'Active: active (running)') ? 1 : 0)];
                }
            } else {
                $data[] = ['name' => $service, 'installed' => 1, 'management' => $mgmt];
            }
        }
        return $data;
    }

    /**
     * Start a service
     * 
     * @return array()
     */
    public function start($service = null)
    {
        if ($service === null) { return; }
        if ($this->utils->checkManagementRunning(false)) {
            $id = $this->hg->managementServiceCommand($service,'start');
            $result = $this->utils->getCommandStatus($id);
            if ($result['exitCode'] == 0) {
                // enable service after successful start
                $id = $this->hg->managementServiceCommand($service,'enable');
                $result = $this->utils->getCommandStatus($id);
                return ['result' => 'success', 'service' => $service];
            }
            if (array_key_exists('output',$result) && $result['output'] == '') {
                if ($service == 'influxdb') {
                    $result['output'] = $this->translator->trans('services.start_failed_influxdb',array(),'admin');
                } else {
                    $result['output'] = $this->translator->trans('services.start_failed',array(),'admin');
                }
            }
            return ['result' => 'error', 'service' => $service, 'msg' => $result['output']];
        }
        return ['result' => 'error', 'service' => $service, 'msg' => $this->translator->trans('management.not_running',array(),'admin')];
    }

    /**
     * Stop a service
     * 
     * @return array()
     */
    public function stop($service = null)
    {
        if ($service === null) { return; }
        if ($this->utils->checkManagementRunning(false)) {
            $id = $this->hg->managementServiceCommand($service,'stop');
            $result = $this->utils->getCommandStatus($id);
            if ($result['exitCode'] == 0) {
                // disable service
                $id = $this->hg->managementServiceCommand($service,'disable');
                $result = $this->utils->getCommandStatus($id);
                return ['result' => 'success', 'service' => $service];
            }
            return ['result' => 'error', 'service' => $service, 'msg' => $result['output']];
        }
        return ['result' => 'error', 'service' => $service, 'msg' => $this->translator->trans('management.not_running',array(),'admin')];
    }

    /**
     * Restart a service
     * 
     * @return array()
     */
    public function restart($service = null)
    {
        if ($service === null) { return; }
        if ($this->utils->checkManagementRunning(false)) {
            $id = $this->hg->managementServiceCommand($service,'restart');
            $result = $this->utils->getCommandStatus($id);
            if ($result['exitCode'] == 0) {
                return ['result' => 'success', 'service' => $service];
            }
            return ['result' => 'error', 'service' => $service, 'msg' => $result['output']];
        }
        return ['result' => 'error', 'service' => $service, 'msg' => $this->translator->trans('management.not_running',array(),'admin')];
    }

    /**
     * Reboot the central
     * 
     * @return none
     */
    public function reboot()
    {
        if ($this->utils->checkManagementRunning(false)) {
            $this->hg->managementReboot();
        }
    }

    /**
     * Set a password for a user (used for ssd)
     * 
     * @return none
     */
    public function setSshPassword($params = null)
    {
        if ($params === null || !array_key_exists('sshPassword',$params) || $params['sshPassword'] == '') {
            return ['result' => 'error', 'message' => $this->translator->trans('services.ssh_password_missing',array(),'admin')];
        }
        if ($this->utils->checkManagementRunning(false)) {
            try {
                $this->hg->managementSetUserPassword('user',$params['sshPassword']);
            } catch (\Homegear\HomegearException $e) {
                return ['result' => 'error', 'message' => $this->translator->trans('services.ssh_password_exception',array('%message%' => $e->getMessage()),'admin')];
            }
            return ['result' => 'success', 'message' => $this->translator->trans('services.ssh_password_success',array(),'admin')];
        }
        return ['result' => 'error', 'message' => $this->translator->trans('management.not_running',array(),'admin')];
    }

    public function setEnv($env = null)
    {
        if ($env === null) { return; }
        if ($env == 'prod') {
            $this->hg->setData('webEnv','env','prod');
        } else {
            $this->hg->deleteData('webEnv');
        }
        return 1;
    }
}
